      subroutine bisect1(xstart,xend,dx,eps,roots,nw)
      USE mod_debug_bisect
      implicit none
      real*8, dimension(*) :: roots
      real*8 xstart,xend,dx,eps
      integer nw


      !* Parameters:
      !* func_zero - external target function for which zeroes are to found
      !* xstart    - startvalue of (x) search interval 
      !* xend      - endvalue of (x) search interval 
      !* dx        - grid value used in locating change of sign
      !*             dx must smaller than the distance bwetween multiple roots
      !* eps       - absolute precision of the roots
      !* roots     - output array to store the roots 
      !* nw        - number of roots found in search interval

      real*8, external :: func_zero

      ! local variables
      integer iter
      double precision x,y,x1,y1,xm,ym
!>>>> logical, parameter :: DEBUG = .false.   (replaced by debug_bisect in mod_debug_bisect)

      !* initialize the number of roots and function evaluation argument
      nw = 0
      x  = xstart

      do  !* search for the next change of sign
        if ( x + dx > xend ) exit      
          y  = func_zero(x)
          x1 = x
          y1 = y
          x  =  x + dx
          y  = func_zero(x)
          if (debug_bisect) then
             write(6,2222) x,y,x1,y1
          end if
2222      format(/,'debug_bisect - searching for change of sign:',/,&
                   'x ..... ',e20.12,/,&
                   'y ..... ',e20.12,/,&
                   'x1 .... ',e20.12,/,&
                   'y1 .... ',e20.12)

          if( y*y1 .le. 0. ) then
              !* change of sign located
              iter = 0
              do  !* refine the search interval - succesive bisection
                  iter = iter + 1

                  if (debug_bisect) then
                    write(6,1111) x,x1,abs(x-x1), eps
                  end if
1111      format(/,'debug_bisect - refining search interval:',/,  &
                   'x ..... ',e20.12,/, &
                   'x1 .... ',e20.12,/, &
                   '|x-x1|  ',e20.12,/, &
                   'eps ... ',e20.12)

                  if( 1d0 + abs(x-x1) == 1d0 ) exit
!>>>>>>>>>>>>     if( abs(x-x1) < eps ) exit
                  xm = ( x + x1 )/2
                  ym = func_zero(xm)
                  if( ym*y1 .le. 0.) then
                     !* root located left 
                       x = xm
                       y = ym
                        
                  else
                     !* root located right
                       x1 = xm
                       y1 = ym
                        
                  end if
              end do  !* end refine the search interval 

              !* convergene: store the root
              nw = nw + 1
              roots(nw) = (x + x1)/2

              if (debug_bisect) then
                write(6,3333) iter, nw, roots(nw)
              end if
3333          format(/,'debug_bisect - convergence:',/,&
                       'iter ..... ',i20,/,            &
                       'nw ....... ',i20,/,            &
                       'roots(nw)  ',e20.12)

              !* def. start search for next change of sign
              x  = roots(nw) + dx/2
                
          end if
      end do

      return
      end subroutine bisect1
