     subroutine compute_alpha(temperature,volume,alpha)
     USE mod_quasi_multistein_param2
     USE mod_constants
     implicit none
     real*8 temperature,volume,alpha

     ! locals
     integer j
     real*8 gamma_v,expx,x,x3,x4,kstatic,kvib,kincompres
     real*8 anharm0,anharm1,sum1,sum2
     real*8 dpdt_v,dpdt_vhar,dpdt_vanh

     ! Grueneisen parameter
     gamma_v = gamma_inf + (gamma_0 - gamma_inf) * (volume/v0)**q0
     ! Compute einstein temperature theta_e(j) and x_e(j)=theta_e/temperature
     call multistein_temperature2(volume,temperature)

     ! harmonic contribution
     sum1 = 0d0
     do j=1,n_oscillators
        expx = exp(x_e(j))
        sum1  = sum1 + vdos(j) * x_e(j)**2 * expx / ( expx - 1d0)**2
     end do
     dpdt_vhar = 3*n_atoms_perf*gas_constant*gamma_v/volume * sum1 

     ! anharmonic contribution
     if ( anharmonicity0 == 0d0 ) then  ! quasi harmonic case
        dpdt_vanh = 0d0
     else                               ! non-zero anharmonic contrib.
        sum1=0d0; sum2=0d0
        do j=1,n_oscillators
          x    = x_e(j)
          x3   = x**3
          x4   = x**4
          expx = exp(x)      
          sum1 = sum1 + vdos(j) *                    &
            ( 3d0*x3*expx**3 * (3d0-x)               &
              -12d0*x4*expx**2 - 3d0*x3*expx*(3d0+x) &
            ) / (expx -1d0)**4
          sum2 = sum2 + vdos(j) * 3d0*x3*expx*(expx+1d0) / (expx-1d0)**3
        end do
        ! volume dependent anharmonicity and its volume derivative
        anharm0 = anharmonicity0 * (volume/v0)**zvol_expon
        anharm1 = anharmonicity0 * (zvol_expon/v0) * (volume/v0)**(zvol_expon-1)
        dpdt_vanh = 0.5d0*n_atoms_perf*gas_constant*temperature *     &
                    ( (gamma_v/volume) * anharm0*sum1 - anharm1*sum2 )
     end if
     
     dpdt_v = dpdt_vhar + dpdt_vanh

     call compute_kincompres(temperature,volume,kstatic,kvib,kincompres)

     alpha = 1d0/kincompres * dpdt_v

     return
     end subroutine compute_alpha
