      subroutine compute_energy(temperature,volume,energy)
      USE mod_quasi_multistein_param2
      USE mod_constants
      implicit none
      real*8 temperature,volume,energy

      ! locals
      integer j
      real*8 anharm,eta,k0_eta,expx,x,x2,x3
      real*8 u_static,u_vib,u_vib_harm,u_vib_anharm
      real*8 a_vib_anharm,entropy_anharm
      real*8 sum1,sum2

      ! compression
      eta = volume / v0_st

      k0_eta = 1.5d0*(k0_prime_st-1d0)*(1d0-eta**(1d0/3d0))

      u_static = 4*k0_st*v0_st/(k0_prime_st-1d0)**2 *         &
                   ( 1d0 + (k0_eta - 1d0 ) * exp( k0_eta )  ) 

      call multistein_temperature2(volume,temperature)

      sum1 = 0d0
      do j=1,n_oscillators
         sum1 = sum1 + vdos(j) * theta_e(j) *  &
               ( 0.5d0 + 1d0/( exp(x_e(j)) - 1d0 ) ) 
      end do

      u_vib_harm = 3*n_atoms_perf*gas_constant * sum1

      ! anharmonic part energy U 
      if ( anharmonicity0 == 0d0 ) then
        u_vib_anharm = 0d0
      else
        ! U = A + TS => U^vib_anh = A^vib_anh + T S^vib_anh 

        ! A^vib_anh - anharm. part of Helmh. free energy - PCM (12)
        sum2=0d0
        do j=1,n_oscillators
          x     = x_e(j)
          x2    = x**2
          expx  = exp(x)
          sum2 = sum2 + vdos(j) * ( 0.25d0*x + 3d0*x2*expx / ( expx - 1d0 )**2 )
        end do
        anharm = anharmonicity0 * (volume/v0)**zvol_expon
        a_vib_anharm = 0.5d0*n_atoms_perf*gas_constant * temperature**2 * anharm*sum2

        ! S^vib_anh - anharm. part of entropy - PCM (28)
        sum2 = 0d0
        do j=1,n_oscillators
          x     = x_e(j)
          x3    = x**3
          expx  = exp(x)
          sum2 = sum2 + vdos(j) * ( 3d0*x3*expx*(expx+1d0) / ( expx - 1d0 )**3 )
        end do
        ! volume dependent anharmonicity
        entropy_anharm = - 0.5d0*n_atoms_perf*gas_constant * temperature*anharm*sum2

        u_vib_anharm = a_vib_anharm + temperature*entropy_anharm
      end if

      u_vib = u_vib_harm + u_vib_anharm

      energy = u_ref + u_static + u_vib

      return
      end subroutine compute_energy
