     subroutine compute_kincompres(temperature,volume,kstatic,kvib,kincompres)
     USE mod_quasi_multistein_param2
     USE mod_constants
     implicit none
     real*8 temperature,volume,kstatic,kvib,kincompres

     ! locals
     integer j
     real*8 gamma_v, eta, dgammadv_t, sum1, expx
     real*8 kvib_har,kvib_anh

     ! compression
     eta = volume / v0_st

     ! Grueneisen parameter
     gamma_v = gamma_inf + (gamma_0 - gamma_inf) * (volume/v0)**q0
     ! Einstein temperature(s)
     call multistein_temperature2(volume,temperature)

     kstatic = k0_st*( 2d0*eta**(-2d0/3d0) - eta**(-1d0/3d0) +              &
                       3d0/2d0*(k0_prime_st-1d0) * (eta**(-1d0/3d0)-1d0)    &
                     ) *                                                    &
                     exp(3d0/2d0*(k0_prime_st-1d0) * (1d0 - eta**(1d0/3d0)))

     dgammadv_t = ( gamma_0 - gamma_inf ) * q0 * (volume/v0)**(q0-1d0) / v0  

     ! harmonic contribution
     sum1 = 0d0
     do j=1,n_oscillators
       expx = exp(x_e(j))
       sum1 = sum1 + vdos(j) *                                  &
              ( ( 0.5d0 + 1d0/(expx - 1d0) ) * x_e(j) *         &
                ( gamma_v**2 + gamma_v - volume*dgammadv_t )    &
               -  gamma_v**2 * x_e(j)**2 * expx / (expx-1d0)**2 &
              )
     end do
     kvib_har = 3*n_atoms_perf*gas_constant*temperature/volume * sum1 
                   
     ! anharmonic contribution
     if ( anharmonicity0 == 0d0 ) then  ! quasi harmonic case
        kvib_anh = 0d0
     else                               ! non-zero anharmonic contrib.
        call compute_kvib_anharm(temperature,volume,gamma_v,dgammadv_t,kvib_anh)
     end if

     kincompres = kstatic + kvib_har + kvib_anh

     return
     end subroutine compute_kincompres
