     subroutine compute_volume(i_pressure,j_temperature,nroots)
     USE mod_quasi_multistein_param2
     USE mod_state_volume
     USE mod_debug_bisect
     implicit none
     integer i_pressure,j_temperature,nroots

     real*8, parameter :: MAXCOMPRES = 0.1d0
     real*8, parameter :: MAXEXPAND  = 3.0d0
!>>> real*8, parameter :: MAXEXPAND  = 2.0d0
     real*8, parameter :: EPS = 0d0  ! (=> machine precision)

     logical, parameter:: debug_nroots_zero=.false.

     integer jroot
     real*8 v_start,v_end,v_delta
     real*8, dimension(10) :: roots
     integer, parameter:: nsub_intervals = 5

     integer jfunc
     integer, parameter :: nfunc=101
     real*8 dvolume
     real*8, allocatable, dimension(:) :: volume_func, func_value
     real*8, external :: func_zero

     ! define volume search window
     v_start = MAXCOMPRES * v0
     v_end   = MAXEXPAND  * v0
     v_delta = (v_end - v_start)/nsub_intervals

     ! solve the EOS for volume V(P,T)
     debug_bisect = .false.
     call bisect1(v_start,v_end,v_delta,eps,roots,nroots)

     if ( nroots==0 ) then
        ! no solution (T > T_max(P)) 
     else if ( nroots == 1 ) then
        !* normal case - single solution for the volume
        volume_table(i_pressure,j_temperature) = roots(1)
!>!>DEBUG>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!>         print *, 'DEBUG compute_volume (case nroots=1): volume_table=', &
!>                  volume_table(i_pressure,j_temperature)
!>!>DEBUG>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
     else if ( nroots == 2 ) then
        !* high temperature and low pressure 
        !  pressure-volume curve has a minimum and 2-roots are found in bisect
        !  the smallest volume is used her
        volume_table(i_pressure,j_temperature) = min(roots(1),roots(2))
     else 
        write(6,9000) nroots,temperature, pressure
        if ( nroots > 0 ) then
           write(6,9100) (jroot,roots(jroot),                      &
                          jroot,roots(jroot)/v0_st, jroot=1,nroots)
        end if
        ! compute - pressure volume curves for debugging
        ! for current temperature
        call pressure_volume_testtemp()

        ! rerun bisect1 with the debug flag set in mod_debug_bisect
        debug_bisect = .true.
        call bisect1(v_start,v_end,v_delta,eps,roots,nroots)
        
        ! debug write table of target function values
        allocate(volume_func(nfunc),func_value(nfunc))
        dvolume = (v_end - v_start)/(nfunc-1)
        do jfunc = 1,nfunc
          volume_func(jfunc) = v_start + (jfunc-1)*dvolume
           func_value(jfunc) = func_zero(volume_func(jfunc))
        end do
        call writdatfile(nfunc,volume_func,func_value,'func_value-volume.dat')

        stop 'error routine compute_volume nroots < 1 or > 2'
     end if  

     return
9000 format(/,'ERROR in procedure compute_volume: iterat. sol. EOS',/, &
            ' # roots (bisect algor.)    ',i15,/,   &
            'evaluation temperature [K]  ',e15.7,/, &
            'evaluation pressure .. [Pa] ',e15.7,/, &
            'P-V curves are dumped in local *.dat files' )
9100 format(/,'Printout multiple roots (volume) :',/,   &
            10('v(',i1,'):      ',e15.7,' [m^3/mol], ', & 
               'v(',i1,')/v0_st:',e15.7,' [-]',:,/      &
              )                                         &
           )
     end subroutine compute_volume
