     subroutine equation_of_state2(volume,temperature,p_static,p_vib,gamma_v,pressure_out)
     USE mod_quasi_multistein_param2
     USE mod_constants
     implicit none
     real*8 volume, temperature,p_static,p_vib,gamma_v, pressure_out

     ! Purpose:
     ! Compute a pressure value by evaluating the thermal equation of state
     ! for a given value of the volume. This routine is called in the iterative
     ! computation of volume for given P,T 

     integer j
     real*8 eta,x,x2,expx,p_vib_har,p_vib_anh
     real*8 anharm0,anharm1,sum1,sum2

     ! compression
     eta = volume / v0_st 

     p_static = 3*k0_st * ( eta**(-2d0/3d0) - eta**(-1d0/3d0) ) *  &
                exp( 1.5d0*(k0_prime_st-1d0)*(1d0-eta**(1d0/3d0)) ) 

     if ( temperature > 0d0 ) then 
       ! compute thermal pressure
       ! Grueneisen parameter
       gamma_v = gamma_inf + (gamma_0 - gamma_inf) * (volume/v0)**q0
  
       ! Einstein temperature(s) theta_e(j) and x_e(j)=theta_e(j)/T
       call multistein_temperature2(volume,temperature)
  
       sum1 = 0d0
       do j=1,n_oscillators       ! assuming uniform gamma=gamma_j
         sum1 = sum1 +  &
                 vdos(j) * ( 0.5d0*theta_e(j) + theta_e(j)/(exp(x_e(j)) - 1d0) )
       end do
       p_vib_har = 3*n_atoms_perf*gas_constant*gamma_v/volume * sum1

       if ( anharmonicity0 == 0d0 ) then  ! quasi harmonic case
          p_vib_anh = 0d0   
       else                               ! anharmonic contrib. thermal pressure
          sum1 = 0d0; sum2=0d0
          do j=1,n_oscillators
            x    = x_e(j)
            x2   = x**2
            expx = exp(x)
            sum1 = sum1 +  vdos(j) *  &
             ( 0.5d0*x2 + &
               ( 3d0*x2*(2-x)*expx**2 - 3d0*x2*(2+x)*expx )/(expx-1d0)**3                                 &
             )
            sum2 = sum2 + vdos(j) * ( 0.25d0*x2 + 3d0*x2 * expx/(expx-1d0)**2 )
          end do
          ! volume dependent anharmonicity and its volume derivative
          anharm0 = anharmonicity0 * (volume/v0)**zvol_expon
          anharm1 = anharmonicity0 * (zvol_expon/v0) * (volume/v0)**(zvol_expon-1) 
          p_vib_anh = 0.5d0*n_atoms_perf*gas_constant*temperature**2 *     &
                      ( (gamma_v/volume) * anharm0*sum1 - anharm1*sum2 )
       end if
       p_vib = p_vib_har + p_vib_anh

     else if ( temperature == 0d0 ) then 
       ! benchmark calc. at T=0 K
       p_vib = 0d0
     else
       print *, 'ERROR proc. equation_of_state T = ',temperature,' <0'
       stop 'ERROR proc. equation_of_state T<0'
     end if

     pressure_out = p_static + p_vib

     return
     end subroutine equation_of_state2
