      subroutine read_quasi_einstein_param
      USE mod_quasi_multistein_param2
      implicit none

      integer, parameter :: LUIN=9, LUOUT=6
      character(LEN=80), parameter :: file_einstein_param='quasi_einstein_param.dat'
      character(LEN=80), parameter :: file_vdos='vdos_param.dat'

      integer j,iformat,n_oscillators_data
      real*8 scalfact_mult_vdos

      open(LUIN,FILE=file_einstein_param)
      read(LUIN,*) n_oscillators  ! # of discrete einstein oscillators in model
      read(LUIN,*) n_atoms_perf   !  0 - # atoms per formula unit
      read(LUIN,*) v0_st          !  1 - volume of the static lattice 
      read(LUIN,*) k0_st          !  2 - isothermal bulk modulus of the static lattice
      read(LUIN,*) k0_prime_st    !  3 - pressure derivative of isothermal bulk modulus (stat.lattice
      read(LUIN,*) v0             !  4 - real (measurable) volume of the lattice at zero Kelvin and zero pressure
      read(LUIN,*) gamma_0        !  5 - Grueneisen parameter at zero Kelvin and zero pressure
      read(LUIN,*) gamma_inf      !  6 - Grueneisen parameter at zero Kelvin and infinite pressure
      read(LUIN,*) q0             !  7 - Mode q-parameter at zero Kelvin and zero pressure
      read(LUIN,*) u_ref          !  8 - Reference energy
      read(LUIN,*) mol_mass       !  9 - Molecular mass
      read(LUIN,*) anharmonicity0 ! 10 - anharmonicity parameter a0 in: a=a0(v/v0)**z
      read(LUIN,*) zvol_expon     ! 11 - anharmonicity parameter  z in: a=a0(v/v0)**z
      close(LUIN)

      write(LUOUT,FMT=1000) file_einstein_param(1:len_trim(file_einstein_param)),   &
                            n_oscillators,n_atoms_perf, v0_st,k0_st,k0_prime_st,v0, &
                            gamma_0,gamma_inf,q0,u_ref,mol_mass,        &
!>>>>>>                     theta0_e(1),gamma_0,gamma_inf,q0,u_ref,mol_mass,        &
                            anharmonicity0,zvol_expon
      call flush(LUOUT)

      if (n_oscillators >= 1) then
         allocate ( x_e(n_oscillators), theta_e(n_oscillators), theta0_e(n_oscillators),  &
                    vdos(n_oscillators) )
         open(LUIN,FILE=file_vdos)
         ! read multi-einstein VDoS data from file
         read(LUIN,*) n_oscillators_data, iformat, scalfact_mult_vdos
         write(LUOUT,1200) file_vdos(1:len_trim(file_vdos)),n_oscillators_data, &
                           iformat, scalfact_mult_vdos
         if ( n_oscillators_data /= n_oscillators) then
            write(LUOUT,9100) n_oscillators, file_einstein_param(1:len_trim(file_einstein_param)), &
                              n_oscillators_data,file_vdos(1:len_trim(file_vdos))
            stop 'ERROR read_quasi_einstein_param (n_oscillators_data)'
         end if
         do j=1,n_oscillators
           if ( iformat == 1 ) then
             read(LUIN,*) theta0_e(j),vdos(j)
           else if (iformat == 2 ) then
             read(LUIN,*) vdos(j),theta0_e(j)
           else
             print *, 'ERROR read_quasi_einstein_param - unknown iformat=',iformat
             stop 'ERROR read_quasi_einstein_param - unknown iformat'
           end if
         end do
         close(LUIN)
         vdos = vdos*scalfact_mult_vdos
         call writdat(n_oscillators,theta0_e,vdos,'vdos.dat')
      else
         write(LUOUT,9000) n_oscillators
         stop 'ERROR read_quasi_einstein_param (n_oscillators)'
      end if

      write(LUOUT,2000) n_oscillators,file_vdos(1:len_trim(file_vdos)), &
                        (j,theta0_e(j),vdos(j),j=1,n_oscillators)

      write(LUOUT,3000) sum(vdos)
      if ( abs(1d0 - sum(vdos)) > 1d-2 ) then
         write(LUOUT,9200)
9200     format(/,'ERROR - read_quasi_einstein_param: sum(vdos) /= 1',/,        &
                'Check scaling input table - use different format switch (?)')
         stop 'ERROR - read_quasi_einstein_param: sum(vdos) /= 1'
      end if
      call flush(LUOUT)

      return
1000  format(/,'Proc. read_quasi_einstein_param - model parameters read from file= ',a,/,                           &
             'n_oscillators  ',  i15,'       (-) # oscillators in model ',/,                                         &
             'n_atoms_perf   ',  i15,'       (-)  0 - # atoms per formula unit',/,                                    &
             'v0_st ........ ',d15.7,' (m^3/mol)  1 - volume of the static lattice',/,                                &
             'k0_st ........ ',d15.7,'      (Pa)  2 - isothermal bulk modulus of the static lattice',/,               &
             'k0_prime_st .. ',d15.7,'       (-)  3 - pressure derivative of isothermal bulk modulus (stat.lattice',/,&
             'v0 ........... ',d15.7,' (m^3/mol)  4 - real (measurable) volume of the lattice at zero Kelvin and ',   &
                                                                                                  'zero pressure',/,&
             'gamma_0 ...... ',d15.7,'       (-)  5 - Grueneisen parameter at zero Kelvin and zero pressure',/,       &
             'gamma_inf .... ',d15.7,'       (-)  6 - Grueneisen parameter at zero Kelvin and infinite pressure',/,   &
             'q0 ........... ',d15.7,'       (-)  7 - Mode q-parameter at zero Kelvin and zero pressure',/,           &
             'u_ref ........ ',d15.7,'   (J/mol)  8 - Reference energy',/,                                            &
             'mol_mass ..... ',d15.7,'  (kg/mol)  9 - Molar mass',/,                                                  &
             'anharmonicity0 ',d15.7,'     (1/K) 10 - a0 in: a=a0*(v/v0)**z',/,                                       &
             'zvol_expon ... ',d15.7,'       (-) 11 -  z in: a=a0*(v/v0)**z')
!>>>>>>>>>>  'theta0_e(1) .. ',d15.7,'       (K)    - (first) einstein temperature',/,                            
1200  format(/,'Reading VDoS data file: ',a,/,                     &
             'n_oscillators_data ...... ',i5,/,                    &
             'iformat (data sequence)   ',i5,/,                    &
             ' 1 - theta0_e(j),vdos(j)  ',/,                       &
             ' 2 - vdos(j),theta0_e(j)  ',/,                       &
             'scalfact_mult_vdos ...... ',d15.7,/,                 &
             ' (=multiplic. factor will be applied to input VDoS)')
2000  format(/,'Proc. read_quasi_einstein_param - print VDoS data for ',i5,' einstein oscillators (file: ',a,')',/, &
               '  j ',2x,'theta0_e(j) /K','      vdos(j)',/,(100(i3,':',2e15.7,/)))
3000  format(/,'Proc. read_quasi_einstein_param - sum(VDoS) = ',e15.7)
9000  format(/,'ERROR read_quasi_einstein_param - illegal # oscillators = ',i5)
9100  format(/,'ERROR read_quasi_einstein_param - inconsistent # oscillators specified in input files:',/, &
             'n_oscillators .......',i5,' file: ',a,/, &
             'n_oscillators_data ..',i5,' file: ',a)
      end subroutine read_quasi_einstein_param
