      program ptdata_emg
      USE mod_state_volume
      USE mod_quasi_multistein_param2
      implicit none

      integer i_pressure,j_temperature,j,nisotherms, nroots, lnflnm
      integer nisothmin,nisothstp,nisothmax  ! spec's subset isotherms => *.dat
      integer nisobarmin,nisobarstp,nisobarmax
      character(LEN=80) filename
      ! 1-D tables
      real*8, allocatable, dimension(:) :: pressure_table
      real*8, allocatable, dimension(:) :: temperature_array, alpha1d, rho1d
      real*8, allocatable, dimension(:) :: volume1d,kstatic,kvib,kincompres1d,cp1d
      real*8, allocatable, dimension(:) :: energy1d,entropy1d,gibbs_energy1d,enthalpy1d

      ! 2-D tables
      real*8, allocatable, dimension(:,:) :: rho,alpha,kincompres,cp
      real*8, allocatable, dimension(:,:) :: gibbs_energy,enthalpy,entropy

      real*8 volume,ks_dummy,kv_dummy, energy
      real*8 press1gpa,deltapressgpa,temper1kelvin,deltatemperkelvin

      character(LEN=80) binfile

      read(5,*) temper_min,temper_max, n_temperature
      if ( n_temperature > 1 ) then
        delta_temperature = ( temper_max - temper_min ) / (n_temperature -1)
      else
        delta_temperature = 0d0
      end if

      read(5,*) pressure_min,pressure_max, n_pressure
      if ( n_pressure > 1 ) then
        delta_pressure    = ( pressure_max - pressure_min ) / (n_pressure -1)
      else
        delta_pressure = 0d0
      end if

      write(6,1000) temper_min,temper_max, n_temperature, delta_temperature, &
                    pressure_min,pressure_max, n_pressure, delta_pressure

      ! spec's subset isotherms written to *.dat file
      read(5,*) nisothmin,nisothstp,nisothmax
      write(6,1100) nisothmin,nisothstp,nisothmax
      ! spec's subset isobars written to *.dat file
      read(5,*) nisobarmin,nisobarstp,nisobarmax
      write(6,1110) nisobarmin,nisobarstp,nisobarmax

      allocate ( pressure_table(n_pressure) )
      allocate ( temperature_array(n_temperature),alpha1d(n_temperature),rho1d(n_temperature))
      allocate ( kstatic(n_temperature), kvib(n_temperature), kincompres1d(n_temperature))
      allocate ( volume1d(n_temperature) )
!>>>>>allocate ( volume1d(n_temperature), theta_e(n_temperature) )
      allocate ( cp1d(n_temperature),energy1d(n_temperature),entropy1d(n_temperature))
      allocate ( gibbs_energy1d(n_temperature),enthalpy1d(n_temperature) )

      allocate ( volume_table(n_pressure,n_temperature) )
      allocate ( rho(n_pressure,n_temperature) )
      allocate ( alpha(n_pressure,n_temperature) )
      allocate ( kincompres(n_pressure,n_temperature), cp(n_pressure,n_temperature) )
      allocate ( gibbs_energy(n_pressure,n_temperature) )
      allocate ( enthalpy(n_pressure,n_temperature), entropy(n_pressure,n_temperature) )

      call read_quasi_einstein_param   ! read material einstein parameters from file
      call flush(6)

      do i_pressure=1, n_pressure   ! loop over druk index i
         pressure = pressure_min + (i_pressure-1)*delta_pressure
         pressure_table(i_pressure) = pressure     ! located in mod_stat_volume
         do j_temperature=1, n_temperature  ! loop over temperatuur index j
            temperature = temper_min + (j_temperature-1)*delta_temperature
            call compute_volume(i_pressure,j_temperature, nroots)   ! molar volume

            if ( nroots == 0 ) then
               ! T > T_max(P) ( > T_melt) no solutions found - copy from previous T
               if ( j_temperature > 1 ) then
                  ! copy solutions from previous temperature
                  volume_table(i_pressure,j_temperature) = volume_table(i_pressure,j_temperature-1)
                         alpha(i_pressure,j_temperature) =        alpha(i_pressure,j_temperature-1)
                    kincompres(i_pressure,j_temperature) =   kincompres(i_pressure,j_temperature-1)
                            cp(i_pressure,j_temperature) =           cp(i_pressure,j_temperature-1)
                  gibbs_energy(i_pressure,j_temperature) = gibbs_energy(i_pressure,j_temperature-1)
                      enthalpy(i_pressure,j_temperature) =     enthalpy(i_pressure,j_temperature-1)
               else
                  print *, 'ERROR nroots == 0 & j_temperature == 1'
                  stop 'main prog: ERROR nroots == 0 & j_temperature == 1'
               end if
            else
               call compute_alpha(temperature,volume_table(i_pressure,j_temperature),            & ! therm. exp.
                                  alpha(i_pressure,j_temperature))
               call compute_kincompres(temperature,volume_table(i_pressure,j_temperature),       & ! incompressibility
                                       ks_dummy,kv_dummy, kincompres(i_pressure,j_temperature))   
               call compute_cp(volume_table(i_pressure,j_temperature),temperature,               &
                               alpha(i_pressure,j_temperature),                                  &
                               kincompres(i_pressure,j_temperature),cp(i_pressure,j_temperature))
               call compute_energy(temperature,volume_table(i_pressure,j_temperature),energy)
               call compute_entropy(temperature,volume_table(i_pressure,j_temperature),          &
                                    entropy(i_pressure,j_temperature))
               call compute_gibbs_energy(temperature,energy,entropy,pressure,                    &
                                         volume_table(i_pressure,j_temperature),                 &
                                         gibbs_energy(i_pressure,j_temperature))
               call compute_enthalpy(energy,pressure,temperature,                                &
                                     volume_table(i_pressure,j_temperature),                     &
                                     enthalpy(i_pressure,j_temperature))
!>!>DEBUG>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
!>               write(6,111) i_pressure,j_temperature,                &
!>                            pressure,temperature,                    &
!>                            volume_table(i_pressure,j_temperature),  &
!>                            alpha(i_pressure,j_temperature),         &
!>                            kincompres(i_pressure,j_temperature),    &
!>                            cp(i_pressure,j_temperature)
!>111         format('DEBUG: i_pressure=',i5,' j_temperature=',i5,/,   & 
!>                   'pressure=',d15.7,' temperature=',d15.7,/,        & 
!>                   'volume .......... ',d15.7,/,                     &
!>                   'alpha ........... ',d15.7,/,                     &
!>                   'kincompres ...... ',d15.7,/,                     &
!>                   'cp .............. ',d15.7)
!>            stop 'STOP END DEBUG'
!>!>DEBUG>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
            end if
         end do
      end do

      ! write binary P,T tables of rho, alpha, c_p 
      ! - write text file with table structure parameters
      nisotherms        = n_temperature 
      press1gpa         = pressure_min / 1d9
      deltapressgpa     = delta_pressure / 1d9
      temper1kelvin     = temper_min
      deltatemperkelvin = delta_temperature
      open(9,FILE='./PTDATA/tabeldatastruct.txt')
      write(9,2000) n_pressure,nisotherms,            &
                    press1gpa,temper1kelvin,         &
                    deltapressgpa,deltatemperkelvin
      close (9)

      ! density - rho
      ! - convert molar volume array into density array
      rho = mol_mass / volume_table
      binfile = './PTDATA/rho_bin'
      call writebin_ptdata_single(binfile,rho,n_pressure,nisotherms)

      ! thermal expansion - alpha
      binfile = './PTDATA/alpha_bin'
      call writebin_ptdata_single(binfile,alpha,n_pressure,nisotherms)

      ! specific heat - cp
      ! - convert from molar value to standard J/K/kg
      cp = cp / mol_mass
      binfile = './PTDATA/cp_bin'
      call writebin_ptdata_single(binfile,cp,n_pressure,nisotherms)

      ! enthalpy - H
      ! - convert from molar value to standard J/kg
      enthalpy = enthalpy / mol_mass
      binfile = './PTDATA/enthalpy_bin'
      call writebin_ptdata_single(binfile,enthalpy,n_pressure,nisotherms)

      ! entropy - S
      ! - convert from molar value to standard J/K/kg
      entropy = entropy / mol_mass
      binfile = './PTDATA/entropy_bin'
      call writebin_ptdata_single(binfile,entropy,n_pressure,nisotherms)

      ! write subset of isotherms
      do j=nisothmin,nisothmax,nisothstp
         ! density
         call numfilename('./isotherms/rho',j,filename)
         filename=filename(1:len_trim(filename))//'.dat'
         call writdatfile(n_pressure,pressure_table,rho(1,j),filename) 

         ! thermal expansion
         call numfilename('./isotherms/alpha',j,filename)
         filename=filename(1:len_trim(filename))//'.dat'
         call writdatfile(n_pressure,pressure_table,alpha(1,j),filename) 

         ! incompressibility
         call numfilename('./isotherms/kincompres',j,filename)
         filename=filename(1:len_trim(filename))//'.dat'
         call writdatfile(n_pressure,pressure_table,kincompres(1,j),filename) 

         ! cp
         call numfilename('./isotherms/cp',j,filename)
         filename=filename(1:len_trim(filename))//'.dat'
         call writdatfile(n_pressure,pressure_table,cp(1,j),filename) 

         ! H
         call numfilename('./isotherms/enthalpy',j,filename)
         filename=filename(1:len_trim(filename))//'.dat'
         call writdatfile(n_pressure,pressure_table,enthalpy(1,j),filename) 

         ! S
         call numfilename('./isotherms/entropy',j,filename)
         filename=filename(1:len_trim(filename))//'.dat'
         call writdatfile(n_pressure,pressure_table,entropy(1,j),filename) 
      end do

      ! write range of isobars from 2-D tables 
      do i_pressure=nisobarmin,nisobarmax,nisobarstp
        do j=1, n_temperature
          temperature = temper_min + (j-1)*delta_temperature
          temperature_array(j) = temperature
          alpha1d(j)        = alpha(i_pressure,j)
          rho1d(j)          = rho(i_pressure,j)
          ! compute separate contrib. of kincompres
          volume1d(j)       = volume_table(i_pressure,j)
          call compute_kincompres(temperature,volume1d(j),kstatic(j),kvib(j),kincompres1d(j))

          cp1d(j)           =      cp(i_pressure,j)
          entropy1d(j)      = entropy(i_pressure,j)
!>>>>>>>  call compute_entropy(temperature,volume_table(i_pressure,j),entropy1d(j))

          call compute_energy(temperature,volume_table(i_pressure,j),energy1d(j))
          gibbs_energy1d(j) = gibbs_energy(i_pressure,j)
          enthalpy1d(j)     = enthalpy(i_pressure,j)
        end do
        call numfilename('./isobars/alpha',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,       alpha1d,filename)
        call numfilename('./isobars/rho',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,         rho1d,filename)
        call numfilename('./isobars/kstatic',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,       kstatic,filename)
        call numfilename('./isobars/kvib',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,          kvib,filename)
        call numfilename('./isobars/kincompres',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,  kincompres1d,filename)
        call numfilename('./isobars/volume',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,      volume1d,filename)
        call numfilename('./isobars/cp',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,          cp1d,filename)
        call numfilename('./isobars/energy',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,      energy1d,filename)
        call numfilename('./isobars/entropy',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,     entropy1d,filename)
        call numfilename('./isobars/gibbs_energy',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,gibbs_energy1d,filename)
        call numfilename('./isobars/enthalpy',i_pressure,filename)
        lnflnm=len_trim(filename); filename=filename(1:lnflnm)//'.dat'
        call writdatfile(n_temperature,temperature_array,    enthalpy1d,filename)
      end do

      print *, "STOP NORMAL PROGRAM EXIT"
      STOP "NORMAL PROGRAM EXIT"
1000  format(/,'Program prog_einstein: printout input parameters read from stdin:',/,&
             'temper_min ...........[K] ',d15.7,/,  &
             'temper_max ...........[K] ',d15.7,/,  &
             'n_temperature ........... ',i15,/,    &
             'delta_temperature (o) [K] ',d15.7,/,  &
             'pressure_min ...... .[Pa] ',d15.7,/,  &
             'pressure_max ........[Pa] ',d15.7,/,  &
             'n_pressure .............. ',i15,/,    &
             'delta_pressur.....(o)[Pa] ',d15.7)
1100  format('range of isotherms written to *.dat files:',/, &
             'nisothmin ..... ',i5,/,                        &
             'nisothstp ..... ',i5,/,                        &
             'nisothmax ..... ',i5)
1110  format('range of isobars written to *.dat files:',/, &
             'nisobarmin ..... ',i5,/,                        &
             'nisobarstp ..... ',i5,/,                        &
             'nisobarmax ..... ',i5)
2000  format(   2i5,' #n_pressure,nisotherms',/,                         &
             d25.14,' #press1gpa         (minimum pressure)',/,          &
             d25.14,' #temper1kelvin     (minimum temperature)',/,       &
             d25.14,' #deltapressgpa     (equid. grid step pressure)',/, &
             d25.14,' #deltatemperkelvin (equid. grid step temperat.)')
      end program ptdata_emg
